// 62256 Interfacing Example using MikroElektronika LV18F board
// by YH NG
// Requires the 44780 LCD for display purposes.
// All pull-ups and pull-downs jumpers are removed.
// Only PORTA LEDs are enabled.

// LCD module connections
sbit LCD_RS at LATB0_bit;
sbit LCD_EN at LATB1_bit;
sbit LCD_D4 at LATB2_bit;
sbit LCD_D5 at LATB3_bit;
sbit LCD_D6 at LATB4_bit;
sbit LCD_D7 at LATB5_bit;

sbit LCD_RS_Direction at TRISB0_bit;
sbit LCD_EN_Direction at TRISB1_bit;
sbit LCD_D4_Direction at TRISB2_bit;
sbit LCD_D5_Direction at TRISB3_bit;
sbit LCD_D6_Direction at TRISB4_bit;
sbit LCD_D7_Direction at TRISB5_bit;


#define  OE          LATJ.LATJ4
#define  WE          LATJ.LATJ5
#define  CE          LATG.LATG0
#define  DATA_PORT   PORTH
#define  DATA_TRIS   TRISH
#define  ADDR_LOW    LATE
#define  ADDR_HIGH   LATC

unsigned char testData[] = {0, 5, 10, 15, 20, 25, 30, 35}; // random numbers to dump into the RAM

void init() {

    TRISA = 0x00;
    TRISB = 0x00;
    TRISC = 0x00;
    TRISE = 0x00;
    TRISH = 0x00;
    TRISG = 0x00;
    TRISJ = 0x00;
    ADCON1.PCFG0 = 1; // all digital
    ADCON1.PCFG1 = 1;
    ADCON1.PCFG2 = 1;
    ADCON1.PCFG3 = 1;
    CCP1CON.CCP1M0 = 0; // comparators disabled
    CCP1CON.CCP1M1 = 0;
    CCP1CON.CCP1M2 = 0;
    CCP1CON.CCP1M3 = 0;
    CCP2CON.CCP2M0 = 0; // comparators disabled
    CCP2CON.CCP2M1 = 0;
    CCP2CON.CCP2M2 = 0;
    CCP2CON.CCP2M3 = 0;
    PORTA = 0x00;
    PORTE = 0x00;

    CE = 1;

}

void write62256(unsigned int address, unsigned char inputData) {

    WE = 1;
    CE = 1;
    OE = 1;

    ADDR_LOW = (address & 0x00ff); // lower 8-bit address
    ADDR_HIGH = (address & 0x7f00) >> 8; // upper 8-bit address
    DATA_TRIS = 0x00; // data is output

    CE = 0;
    OE = 0;
    WE = 0;

    asm nop   // wait more than 70ns for data to appear in the port
    DATA_PORT = inputData;

    OE = 1;
    WE = 1;
    CE = 1;

    DATA_PORT = 0x00;
}

unsigned char read62256(unsigned int address) {

    unsigned char tempData;

    WE = 1;
    CE = 1;
    OE = 1;

    ADDR_LOW = (address & 0x00ff); // lower 8-bit address
    ADDR_HIGH = (address & 0x7f00) >> 8; // upper 8-bit address
    DATA_TRIS = 0xff; // data is input

    CE = 0;
    OE = 0;

    asm nop  // wait more than 70ns for data to appear in the port
    tempData = DATA_PORT;

    OE = 1;
    WE = 1;
    CE = 1;

    return tempData;

    DATA_PORT = 0x00;

}

void check62256() {

    unsigned int i = 0;              // walking one count
    unsigned int j = 0;              // address count
    unsigned char addrErrorTxt[7];   // address where error occurs
    unsigned char tempTestData;      // temporary buffer for testing register

    for (i = 0; i < 8; i++)          // dump first 8 bytes of data into the 62256...
        write62256(i, testData[i]);

    delay_ms(100);

    for (i = 0; i < 8; i++) {        // ... and then see whether it returns the same exact data.
        tempTestData = read62256(i);
        if (tempTestData != testData[i]) {
            Lcd_Out(2, 1, "SRAM Not Present");  // if data returned does not match, halt program, and notify "SRAM not present".
            while (1) {
                PORTA.RA4 ^= 1;
                delay_ms(250);
            }
        }
    }
    
    tempTestData = 0;

    for (j = 0; j < 32768; j++) {      // For all 32768 bytes.

        for (i = 0; i < 8; i++) {      // Every iteration shifts one bit to the left
                                       // to test each bit in the byte to see whether it's working or not.
            write62256(j, 1 << i);
            //delay_us(1);
            tempTestData = read62256(j);
            if (tempTestData != (1 << i)) {  // if data from RAM does not match the data from the system
                Lcd_Cmd(_LCD_CLEAR);         // show address and halt system, notify "SRAM Error"
                Lcd_Out(1, 1, "SRAM Error");
                IntToStr(j, addrErrorTxt);
                Lcd_Out(2, 1, addrErrorTxt);
                while (1) {
                    PORTA.RA5 ^= 1;
                    delay_ms(250);
                }

            }

        }

    }

}

void main() {

    init();
    delay_ms(1);
    Lcd_Init();
    delay_ms(1);
    Lcd_Cmd(_LCD_CLEAR); // Clear display.
    Lcd_Cmd(_LCD_CURSOR_OFF);
    Lcd_Out(1, 1, "Testing SRAM");
    check62256();
    Lcd_Out(2, 1, "SRAM is OK"); // All pass.
    PORTA.RA3 = 1;

    while (1);

}